<?php

namespace NoccyLabs\React\Http\Attributes;

use Attribute;
use NoccyLabs\React\Http\Security\Exception\SecurityException as ExceptionSecurityException;
use NoccyLabs\React\Http\Security\SecurityException;
use Psr\Http\Message\ServerRequestInterface;

#[Attribute(Attribute::TARGET_METHOD)]
class Route
{
    private string $regex;

    private array $pathSlots = [];

    public function __construct(
        private string $path,
        private ?string $name=null,
        private array $methods=["GET"],
        private array $requireRoles=[]
    ) {
        if ($name) { $this->name = $path; }
        $this->regex = '<^'.preg_replace_callback('<(:([^/]+))>', function ($match) {
            $this->pathSlots[] = $match[2];
            return '([^/]+?)';
        }, $path).'$>';
    }

    public function getName(): string
    {
        return $this->name;
    }

    public function getPath(): string
    {
        return $this->path;
    }

    public function matches(ServerRequestInterface $request): bool
    {
        $path = $request->getUri()->getPath();
        if (!preg_match($this->regex, $path, $m)) {
            return false;
        }
        if (!in_array($request->getMethod(), $this->methods)) {
            return false;
        }
        if (count($this->requireRoles) > 0) {
            $user = $request->getAttribute("user");
            if (!$user) {
                throw new ExceptionSecurityException("Access denied: Unauthorized");
            }
            foreach ($this->requireRoles as $role) {
                if (!in_array($role, $user->getRoles())) {
                    throw new ExceptionSecurityException("Access denied: Unauthorized");
                }
            }
        }
        return true;
    }

    public function resolve(ServerRequestInterface $request): ?array
    {
        $path = $request->getUri()->getPath();
        if (preg_match($this->regex, $path, $m)) {
            $params = array_combine($this->pathSlots, array_slice($m,1));
            return $params;
        }
        return null;
    }


}