<?php

namespace NoccyLabs\React\Http\Handler;

use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use React\Http\Message\Response;
use React\Promise\PromiseInterface;
use React\Stream\ReadableResourceStream;

class StaticHandler
{
    public static $contentTypes = [
        '.js' => 'text/javascript',
        '.htm' => 'text/html',
        '.html' => 'text/html',
        '.txt' => 'text/plain',
        '.jpg' => 'image/jpeg',
        '.png' => 'image/png',
        '.json' => 'application/json',
    ];

    public function __construct(private string $root)
    {
        
    }

    public function __invoke(ServerRequestInterface $request, ?callable $next = null): ResponseInterface|PromiseInterface
    {
        $path = $request->getUri()->getPath();
        $real = realpath($this->root."/".$path);
        if ($real && str_starts_with($real, $this->root) && file_exists($real)) {
            $contentType = $this->findContentType($real);
            return new Response(Response::STATUS_OK, [
                'Content-Type' => $contentType,
            ], new ReadableResourceStream(fopen($real, "rb")));
        }

        if (is_callable($next))
            return $next($request);

        return Response::plaintext("Not found")->withStatus(404);
    }

    private function findContentType(string $filename): string
    {
        $ext = ".".strtolower(pathinfo($filename, PATHINFO_EXTENSION));
        if (array_key_exists($ext, self::$contentTypes)) {
            return self::$contentTypes[$ext];
        }
        return "application/octet-stream";
    }
}
