<?php

namespace NoccyLabs\React\Http\Middleware;

use NoccyLabs\React\Http\Routing\RouteCollection;
use NoccyLabs\React\Http\Security\Exception\SecurityException;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use React\Http\Message\Response;
use React\Promise\PromiseInterface;

class RoutingMiddleware
{
    public function __construct(private RouteCollection $routes)
    {

    }

    public function __invoke(ServerRequestInterface $request, ?callable $next = null): ResponseInterface|PromiseInterface
    {
        try {
            $request = $this->routes->matchRoute($request);
        }
        catch (SecurityException $e) {
            return Response::plaintext($e->getMessage())->withStatus(Response::STATUS_FORBIDDEN);
        }

        if (!$request->getAttribute("route")) {
            return Response::plaintext("Not found")->withStatus(Response::STATUS_NOT_FOUND);
        }

        if (is_callable($next))
            return $next($request);

        return Response::plaintext("Not found")->withStatus(404);
    }
}
