<?php

namespace NoccyLabs\React\Http\Routing;

use NoccyLabs\React\Http\Attributes\Route;
use Psr\Http\Message\ServerRequestInterface;
use ReflectionClass;

class RouteCollection
{

    private array $routes = [];

    public function addController(object $class): void
    {
        $reflection = new ReflectionClass($class);
        foreach ($reflection->getMethods() as $method) {
            if (!$method->isPublic()) continue;
            $attr = $method->getAttributes(Route::class);
            $attr = reset($attr);
            if (!$attr) continue;
            $this->addRoute($attr->newInstance(), $method->getClosure($class));
        }
    }

    public function addRoute(Route $route, mixed $handler): void
    {
        $this->routes[] = [ $route, $handler ];
    }

    public function matchRoute(ServerRequestInterface $request): ServerRequestInterface
    {
        foreach ($this->routes as [ $route, $handler ]) {
            if (!$route->matches($request)) continue;
            $matched = $route->resolve($request);
            return $request
                ->withAttribute("route", $route)
                ->withAttribute("handler", $handler)
                ->withAttribute("route-data", $matched)
                ;
        }
        return $request;
    }
}