<?php

namespace NoccyLabs\React\Http\Security;

use NoccyLabs\React\Http\Security\Exception\InvalidCredentialsException;
use NoccyLabs\React\Http\Security\Exception\UserNotFoundException;

interface UserManagerInterface
{
    /**
     * Get user by user id
     *
     * @param int $userId
     * @return UserInterface
     * @throws UserNotFoundException if the user can not be found
     * @throws InvalidCredentialsException if the credentials are invalid
     */
    public function getUser(int $userId): UserInterface;

    /**
     * Get user by username
     *
     * @param string $username
     * @return UserInterface
     * @throws UserNotFoundException if the user can not be found
     * @throws InvalidCredentialsException if the credentials are invalid
     */
    public function getUserByUsername(string $username): UserInterface;

    /**
     * Get user by active token
     *
     * @param string $token
     * @return UserInterface
     * @throws UserNotFoundException if the user can not be found
     * @throws InvalidCredentialsException if the credentials are invalid
     */
    public function getUserByToken(string $token): UserInterface;

    /**
     * Retrieve all users
     *
     * @return array<int,UserInterface>
     */
    public function getAllUsers(): array;

    /**
     * Delete a user
     *
     * @param UserInterface $user
     * @return void
     */
    public function deleteUser(UserInterface $user): void;

    /**
     * Update an existing user
     *
     * @param UserInterface $user
     * @return void
     */
    public function updateUser(UserInterface $user): void;

    /**
     * Create a new user
     *
     * @param UserInterface $user
     * @return void
     */
    public function createUser(UserInterface $user): void;

}

