<?php

namespace NoccyLabs\React\Http\Session;

use ArrayAccess;
use Psr\Log\LoggerInterface;

class Session implements ArrayAccess, SessionInterface
{
    readonly private string $sessionId;

    private array $data = [];

    private ?int $created = null;

    private ?int $lastActive = null;

    private bool $dirty = false;

    public function __construct(?string $sessionId, array $data = [])
    {
        $this->data = $data;
        $this->sessionId = $sessionId ?? hash("sha1", uniqid(more_entropy:true));
        $this->created = time();
    }

    public function getId(): string
    {
        return $this->sessionId;
    }

    public function getAge(): int
    {
        return time() - $this->created;
    }

    public function getStale(): int
    {
        return time() - $this->lastActive;
    }

    public function isDirty(): bool
    {
        return $this->dirty;
    }

    public function markClean(): void
    {
        $this->dirty = false;
    }

    public function offsetExists(mixed $offset): bool
    {
        $this->lastActive = time();
        return isset($this->data[$offset]);
    }

    public function offsetGet(mixed $offset): mixed
    {
        $this->lastActive = time();
        return $this->data[$offset]??null;
    }

    public function offsetSet(mixed $offset, mixed $value): void
    {
        $this->data[$offset] = $value;
        $this->lastActive = time();
        $this->dirty = true;
    }

    public function offsetUnset(mixed $offset): void
    {
        if (!array_key_exists($offset, $this->data))
            return;
        unset($this->data[$offset]);
        $this->lastActive = time();
        $this->dirty = true;
    }
    
}

