<?php

namespace NoccyLabs\React\Http\Session;

use ArrayAccess;
use Psr\Log\LoggerInterface;

class SessionManager implements SessionManagerInterface
{
    private array $sessions = [];
    
    public function __construct(private LoggerInterface $logger)
    {
        
    }

    public function createSession(): SessionInterface
    {
        $session = new Session(null);
        $this->sessions[$session->getId()] = $session;
        //$this->logger->debug("session: Created new {$session->getId()}");
        return $session;
    }

    public function getSession(string $sessionId): SessionInterface
    {
        if (array_key_exists($sessionId, $this->sessions)) {
            $session = $this->sessions[$sessionId];
            //$this->logger->debug("session: Retrieved {$session->getId()}");
        } else {
            $session = new Session(null);
            $this->sessions[$session->getId()] = $session;
            //$this->logger->debug("session: Recreated {$sessionId} -> {$session->getId()}");
        }
        return $session;
    }

    public function flush(): void
    {
        foreach ($this->sessions as $id=>$session) {
            if ($session->isDirty()) {
                // TODO flush to disk
                $session->markClean();
            }
            if ($session->getStale() > 900) {
                $this->logger->debug("Dropped stale session: {$session->getId()}");
                unset($this->sessions[$id]);
            }
        }
    }

}

