<?php

namespace NoccyLabs\React\Http\Middleware;

use PHPUnit\Framework\Attributes\CoversClass;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Log\LoggerInterface;
use Psr\Log\LoggerTrait;
use React\Http\Message\Response;
use React\Http\Message\ServerRequest;
use Stringable;

#[CoversClass(LoggingMiddleware::class)]
class LoggingMiddlewareTest extends \PHPUnit\Framework\TestCase
{
    
    public function testRequestsAreLogged()
    {
        $logger = new TestLogger();
        $middleware = new LoggingMiddleware($logger);

        $request = new ServerRequest("GET", "/", serverParams:['REMOTE_ADDR'=>"127.0.0.1"]);

        $handler = function (ServerRequestInterface $request) {
            return new Response();
        };

        $response = $middleware($request, $handler);

        $this->assertEquals(1, count($logger->logged));
    }

}

class TestLogger implements LoggerInterface
{
    public array $logged = [];

    use LoggerTrait;

    public function log($level, string|Stringable $message, array $context = []): void
    {
        $this->logged[] = $level.": ".$message;
    }
}