<?php

namespace NoccyLabs\React\Http\Middleware;

use NoccyLabs\React\Http\Attributes\Route;
use NoccyLabs\React\Http\Routing\RouteCollection;
use NoccyLabs\React\Http\Session\SessionInterface;
use NoccyLabs\React\Http\Session\SessionManager;
use PHPUnit\Framework\Attributes\CoversClass;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Log\LoggerInterface;
use Psr\Log\LoggerTrait;
use React\Http\Message\Response;
use React\Http\Message\ServerRequest;
use Stringable;

#[CoversClass(RoutingMiddleware::class)]
class RoutingMiddlewareTest extends \PHPUnit\Framework\TestCase
{
    
    public function testRoutesAreMatched()
    {
        $logger = new class implements LoggerInterface {
            use LoggerTrait;
            public function log($level, string|Stringable $message, array $context = []): void
            {
            }
        };

        /** @var ServerRequestInterface|null */
        $receivedRequest = null;

        $routes = new RouteCollection();
        $route = new Route("/", "home");
        $routes->addRoute($route, function () {});
        
        $next = function (ServerRequestInterface $request) use (&$receivedRequest) {
            $receivedRequest = $request;
            return new Response();
        };

        $middleware = new RoutingMiddleware($routes);

        $request = new ServerRequest("GET", "/", serverParams:['REMOTE_ADDR'=>"127.0.0.1"]);

        $response = $middleware($request, $next);

        $this->assertInstanceOf(ResponseInterface::class, $response);
        $this->assertEquals(200, $response->getStatusCode());
        $this->assertEquals($route, $receivedRequest->getAttribute("route"));
    }

}
